@@ -5,17 +5,19 @@
 Program:
     Flask-Docs
 Version:
-    0.6.2
+    0.6.7
 History:
     Created on 2018/05/20
-    Last modified on 2021/10/16
+    Last modified on 2022/04/02
 Author:
     kwkw
 """
 
 import copy
+import inspect
 import logging
 import os
+from collections import OrderedDict
 from functools import wraps
 
 from flask import Blueprint, current_app, jsonify, request
@@ -43,6 +45,22 @@ class ApiDoc(object):
     with open(os.path.join(APP_TEMPLATES, "js_template_local.html"), "r") as h:
         JS_TEMPLATE_LOCAL = h.read()
 
+    LOCATIONS = {
+        "args": "query",
+        "form": "formData",
+        "headers": "header",
+        "json": "body",
+        "values": "query",
+        "files": "formData",
+    }
+
+    PY_TYPES = {
+        int: "integer",
+        str: "string",
+        bool: "boolean",
+        float: "number",
+    }
+
     def __init__(self, app=None, title="Api Doc", version="1.0.0", description=""):
         if app is not None:
             self.init_app(app, title, version, description)
@@ -64,6 +82,8 @@ class ApiDoc(object):
             "API_DOC_METHODS_LIST", ["GET", "POST", "PUT", "DELETE", "PATCH"]
         )
         app.config.setdefault("API_DOC_PASSWORD_SHA2", "")
+        app.config.setdefault("API_DOC_AUTO_GENERATING_ARGS_MD", False)
+        app.config.setdefault("API_DOC_ALL_MD", True)
 
         with app.app_context():
             self._check_value_type(
@@ -81,7 +101,15 @@ class ApiDoc(object):
                 ],
                 str,
             )
-            self._check_value_type(["API_DOC_ENABLE", "API_DOC_CDN"], bool)
+            self._check_value_type(
+                [
+                    "API_DOC_ENABLE",
+                    "API_DOC_CDN",
+                    "API_DOC_AUTO_GENERATING_ARGS_MD",
+                    "API_DOC_ALL_MD",
+                ],
+                bool,
+            )
             self._check_value_type(
                 [
                     "API_DOC_MEMBER",
@@ -158,27 +186,27 @@ class ApiDoc(object):
         data_dict = {}
 
         for rule in current_app.url_map.iter_rules():
-            func = current_app.view_functions[rule.endpoint]
+            cls = current_app.view_functions[rule.endpoint]
 
-            if not hasattr(func, "view_class"):
+            if not hasattr(cls, "view_class"):
                 continue
 
-            if func.methods is None:
+            if cls.methods is None:
                 continue
 
-            c_name = func.view_class.__name__
+            c_name = cls.view_class.__name__
 
             if c_name in current_app.config["API_DOC_RESTFUL_EXCLUDE"]:
                 continue
 
-            c_doc = self._clean_doc(self._get_api_doc(func))
+            c_name_extra = self._get_first_line_of_doc(self._get_api_doc(cls))
 
-            if c_doc and c_doc != current_app.config["API_DOC_NO_DOC_TEXT"]:
-                c_name = "{}({})".format(c_name, c_doc)
+            if c_name_extra:
+                c_name = "{}({})".format(c_name, c_name_extra)
 
             data_dict.setdefault(c_name, {"children": []})
 
-            for method in func.methods:
+            for method in cls.methods:
                 if method not in current_app.config["API_DOC_METHODS_LIST"]:
                     continue
 
@@ -190,7 +218,7 @@ class ApiDoc(object):
                 }
 
                 self._add_api_data(
-                    data_dict, api_data, getattr(func.view_class, method.lower())
+                    data_dict, api_data, getattr(cls.view_class, method.lower())
                 )
 
             if data_dict[c_name]["children"] == []:
@@ -231,8 +259,10 @@ class ApiDoc(object):
                     if r in current_app.config["API_DOC_METHODS_LIST"]
                 ]
             )
-            if method:
-                url = "{}\t[{}]".format(url, "\t".join(method.split(" ")))
+            if not method:
+                continue
+
+            url = "{}\t[{}]".format(url, "\t".join(method.split(" ")))
 
             api_data = {
                 "url": url,
@@ -243,11 +273,11 @@ class ApiDoc(object):
 
             self._add_api_data(data_dict, api_data, func)
 
-        for k in copy.deepcopy(data_dict):
-            if data_dict[k]["children"] == []:
-                data_dict.pop(k)
+        for bp_name in copy.deepcopy(data_dict):
+            if data_dict[bp_name]["children"] == []:
+                data_dict.pop(bp_name)
             else:
-                data_dict[k]["children"].sort(key=lambda x: x["name"])
+                data_dict[bp_name]["children"].sort(key=lambda x: x["name"])
 
         return data_dict
 
@@ -277,10 +307,15 @@ class ApiDoc(object):
             doc = self._get_api_doc(func)
 
             (
-                api_data["doc"],
                 api_data["name_extra"],
+                api_data["doc"],
                 api_data["doc_md"],
-            ) = self._get_doc_name_extra_doc_md(doc)
+            ) = self._split_doc(doc)
+
+            if current_app.config["API_DOC_AUTO_GENERATING_ARGS_MD"]:
+                args_md = self._get_args_md(func)
+                if args_md:
+                    api_data["doc_md"] = "\n".join([args_md, api_data["doc_md"]])
 
         except Exception as e:
             logger.error(
@@ -298,26 +333,22 @@ class ApiDoc(object):
     def _get_api_doc(self, func):
         func_doc = func.__doc__
         if func_doc:
-            return func_doc.replace("\t", "    ")
+            return func_doc.replace("\t", " " * 4)
         else:
-            return current_app.config["API_DOC_NO_DOC_TEXT"]
+            return ""
 
-    def _clean_doc(self, doc_src):
-        return (
-            doc_src.split("\n\n")[0]
-            .split("\n")[0]
-            .strip(" ")
-            .strip("\n\n")
-            .strip(" ")
-            .strip("\n")
-            .strip(" ")
-        )
+    def _clean_str(self, str):
+        return str.strip(" ").strip("\n\n").strip(" ").strip("\n").strip(" ")
 
-    def _get_doc_name_extra_doc_md(self, doc_src):
-        doc = doc_src.split("@@@")[0]
+    def _get_first_line_of_doc(self, doc_src):
+        return self._clean_str(doc_src.split("\n\n")[0].split("\n")[0])
 
-        if doc != current_app.config["API_DOC_NO_DOC_TEXT"]:
-            name_extra = self._clean_doc(doc)
+    def _split_doc(self, doc_src):
+        doc_src_split = doc_src.split("@@@")
+        doc = doc_src_split[0]
+
+        if doc:
+            name_extra = self._get_first_line_of_doc(doc)
         else:
             name_extra = ""
 
@@ -330,17 +361,22 @@ class ApiDoc(object):
             .rstrip(" ")
         )
 
-        if doc == "":
-            doc = current_app.config["API_DOC_NO_DOC_TEXT"]
-
-        if len(doc_src.split("@@@")) >= 2:
-            doc_md = doc_src.split("@@@")[1].strip(" ")
-            space_count = doc_src.split("@@@")[0].split("\n")[-1].count(" ")
-            doc_md = "\n".join(doc_md.split("\n" + " " * space_count))
+        if len(doc_src_split) >= 2:
+            doc_md = doc_src_split[1].strip(" ")
+        elif doc and current_app.config["API_DOC_ALL_MD"]:
+            doc_md = doc.strip(" ")
+            doc = ""
         else:
             doc_md = ""
 
-        return doc, name_extra, doc_md
+        if doc_md:
+            space_count = doc_src_split[0].split("\n")[-1].count(" ")
+            doc_md = "\n".join(doc_md.split("\n" + " " * space_count))
+
+        if doc == "":
+            doc = current_app.config["API_DOC_NO_DOC_TEXT"]
+
+        return name_extra, doc, doc_md
 
     def _check_value_type(self, data_packages, type, data_type="config"):
         for d in data_packages:
@@ -355,6 +391,155 @@ class ApiDoc(object):
                     )
                 )
 
+    def _get_argument(self, func):
+        func_source = inspect.getsource(func)
+        func_doc = func.__doc__
+        if func_doc:
+            func_source = func_source.replace(func_doc, "")
+        func_source = func_source.replace("\t", " " * 4)
+
+        argument_list = func_source.split("add_argument")
+
+        return argument_list
+
+    def _clean_argument(self, argument_source):
+        argument_data = ""
+        stack = []
+        index = 0
+        for char in argument_source:
+            if char == "(":
+                stack.append(char)
+            elif char == ")":
+                stack.pop()
+            index += 1
+            if not stack:
+                index -= 1
+                argument_data = argument_source[1:index]
+                break
+
+        return self._clean_str(argument_data)
+
+    def _parse_argument(self, argument_data):
+        def _run_eval(eval_str):
+            argument_dict = {}
+            try:
+                argument_dict = eval(eval_str)
+            except Exception as e:
+                e_str = str(e)
+                logger.error(e_str)
+                if "is not defined" in e_str:
+                    err_str = e_str.split("'")[1]
+                    argument_dict = _run_eval(eval_str.replace(err_str, ""))
+                else:
+                    return argument_dict
+
+            return argument_dict
+
+        args_dict = {}
+        arg_list = argument_data.split(",")
+        if len(arg_list) < 1 or not arg_list[0].strip():
+            return args_dict
+
+        name = arg_list[0].strip("'").strip('"').strip()
+        args_dict = OrderedDict(
+            name=name,
+            location="",
+            type="",
+            required="",
+            nullable="",
+            default="",
+            help="",
+        )
+        eval_str = "dict({})".format(",".join(arg_list[1:]).strip())
+        argument_dict = _run_eval(eval_str)
+        if not argument_dict:
+            return args_dict
+
+        args_dict["required"] = "False"
+        args_dict["nullable"] = "True"
+
+        for key, value in argument_dict.items():
+            if key not in args_dict:
+                continue
+            if key == "location":
+                args_dict[key] = ApiDoc.LOCATIONS.get(value, "")
+            elif key == "type":
+                args_dict[key] = ApiDoc.PY_TYPES.get(value, "")
+            else:
+                args_dict[key] = str(value)
+
+        return args_dict
+
+    def _get_restx_argument(self, func):
+        return getattr(func, "__apidoc__", {}).get("expect", [])
+
+    def _clean_restx_argument(self, expect_source):
+        argument_data_list = []
+
+        try:
+            from flask_restx.reqparse import RequestParser
+        except ImportError:  # pragma: no cover
+            return argument_data_list  # pragma: no cover
+
+        if isinstance(expect_source, RequestParser):
+            argument_data_list = expect_source.__schema__
+
+        return argument_data_list
+
+    def _parse_restx_argument(self, argument_data_list):
+        args_list = []
+
+        if not argument_data_list:
+            return args_list
+
+        for argument_data in argument_data_list:
+            args_list.append(
+                OrderedDict(
+                    name=argument_data.get("name", ""),
+                    location=argument_data.get("in", ""),
+                    type=argument_data.get("type", ""),
+                    required=str(argument_data.get("required", "False")),
+                    nullable=str(argument_data.get("nullable", "")),
+                    default=str(argument_data.get("default", "")),
+                    help=argument_data.get("description", ""),
+                )
+            )
+
+        return args_list
+
+    def _get_args_md(self, func):
+        args_dict_list = []
+
+        expect_list = self._get_restx_argument(func)
+        for expect in expect_list:
+            argument_data_list = self._clean_restx_argument(expect)
+            args_list = self._parse_restx_argument(argument_data_list)
+            if not args_list:
+                continue
+            args_dict_list.extend(args_list)
+
+        argument_list = self._get_argument(func)
+        for argument_source in argument_list:
+            argument_data = self._clean_argument(argument_source)
+            args_dict = self._parse_argument(argument_data)
+            if not args_dict:
+                continue
+            args_dict_list.append(args_dict)
+
+        args_md_list = []
+        for args_dict in args_dict_list:
+            if not args_md_list:
+                args_md_list.append("### args")
+                args_md_list.append(
+                    "|" + "|".join(arg_key for arg_key in args_dict.keys()) + "|"
+                )
+                args_md_list.append("|---" * len(args_dict) + "|")
+            args_md_list.append(
+                "|" + "|".join(arg_value for arg_value in args_dict.values()) + "|"
+            )
+
+        return "\n".join(args_md_list)
+
     def _unauthorized(self):
         response = jsonify({"error": "unauthorized"})
         response.status_code = 401
